d0c8380650
* Add functional test for barrier_all_on_stream
* Add rocshmem_barrier_all_on_stream support for GDA and RO backends
Implements rocshmem_barrier_all_on_stream operation for
GPU Direct Access and Reverse Offload backends.
Previously, rocshmem_barrier_all_on_stream was only supported for IPC backend.
* Add functional test for rocshmem_broadcastmem_on_stream
* Add host-side rocshmem_broadcastmem_on_stream API
Implement stream-based broadcast collective operation
- Add rocshmem_broadcastmem_on_stream host API and kernel implementation
- Add functional test TeamBroadcastmemOnStreamTester with multi-stream
support and correctness verification
- Use per-workgroup contexts to avoid contention across parallel streams
API:
rocshmem_broadcastmem_on_stream(team, dest, source, nelems, pe_root, stream)
* Add functional test for rocshmem_getmem_on_stream
* Add host-side rocshmem_getmem_on_stream API
Implement stream-based point-to-point RMA get operation
- Add rocshmem_getmem_on_stream host API and kernel implementation
- Support for asynchronous getmem operations on HIP streams
- Add backend support for GDA, RO, and IPC contexts
- Use work-group collective getmem for efficient memory transfer
API:
rocshmem_getmem_on_stream(dest, source, nelems, pe, stream)
(AI Assist)
* Add host-side rocshmem_putmem_on_stream API
- Add rocshmem_putmem_on_stream for asynchronous remote writes
- Support for concurrent RMA operations on HIP streams
- Add backend support for GDA, RO, and IPC contexts
- Use work-group device collective operation
API:
rocshmem_putmem_on_stream(dest, source, bytes, pe, stream)
(AI Assist)
* Add functional test for rocshmem_putmem_on_stream
* Add host-side rocshmem_putmem_signal_on_stream API
Enables asynchronous putmem operations with signaling on HIP streams.
The implementation includes:
- Kernel wrapper rocshmem_putmem_signal_kernel
- Host interface putmem_signal_on_stream method
- Context layer support across all backends (IPC, GDA, RO)
- Public API
Function signature:
void rocshmem_putmem_signal_on_stream(void *dest, const void *source,
size_t bytes, uint64_t *sig_addr,
uint64_t signal, int sig_op,
int pe, hipStream_t stream);
* Add functional test for rocshmem_putmem_signal_on_stream
* Add host-side rocshmem_signal_wait_until_on_stream API
Enables asynchronous signal wait operations on HIP streams.
The implementation includes:
- Kernel wrapper rocshmem_signal_wait_until_kernel
- Host interface signal_wait_until_on_stream method
- Context layer support across all backends (IPC, GDA, RO)
- Native uint64_t support in wait_until API (generated from P2P_SYNC.py)
Function signature:
void rocshmem_signal_wait_until_on_stream(uint64_t *sig_addr, int cmp,
uint64_t cmp_value,
hipStream_t stream);
(AI Assist)
* Add functional test for rocshmem_signal_wait_until_on_stream
* Add documentation for stream API functions
This commit adds API documentation for the following host-side
stream functions:
- rocshmem_barrier_all_on_stream (collective routines)
- rocshmem_broadcastmem_on_stream (collective routines)
- rocshmem_getmem_on_stream (RMA operations)
- rocshmem_putmem_on_stream (RMA operations)
- rocshmem_putmem_signal_on_stream (signaling operations)
- rocshmem_signal_wait_until_on_stream (point-to-point sync)
The documentation includes function signatures, parameter descriptions,
and detailed explanations of asynchronous behavior and stream handling.
(AI Assist)
* Rename "bytes" -> "nelems"
* Add "_TEST_" to the variables used in tests
* Remove incorrect hipStreamDefault usage
hipStreamDefault is not a default stream. This is a flag.
If stream == nullptr, then just pass it to kernel. It will launch the kernel on the default stream
241 baris
8.7 KiB
C++
241 baris
8.7 KiB
C++
/******************************************************************************
|
|
* Copyright (c) Advanced Micro Devices, Inc. All rights reserved.
|
|
*
|
|
* SPDX-License-Identifier: MIT
|
|
*
|
|
* Permission is hereby granted, free of charge, to any person obtaining a copy
|
|
* of this software and associated documentation files (the "Software"), to
|
|
* deal in the Software without restriction, including without limitation the
|
|
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
|
|
* sell copies of the Software, and to permit persons to whom the Software is
|
|
* furnished to do so, subject to the following conditions:
|
|
*
|
|
* The above copyright notice and this permission notice shall be included in
|
|
* all copies or substantial portions of the Software.
|
|
*
|
|
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
|
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
|
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
|
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
|
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
|
|
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
|
|
* IN THE SOFTWARE.
|
|
*****************************************************************************/
|
|
|
|
#include "team_broadcastmem_on_stream_tester.hpp"
|
|
|
|
#include <rocshmem/rocshmem.hpp>
|
|
#include <hip/hip_runtime.h>
|
|
#include <cstring>
|
|
#include <cassert>
|
|
#include <vector>
|
|
|
|
/******************************************************************************
|
|
* HOST TESTER CLASS METHODS
|
|
*****************************************************************************/
|
|
TeamBroadcastmemOnStreamTester::TeamBroadcastmemOnStreamTester(TesterArguments args)
|
|
: Tester(args) {
|
|
my_pe = rocshmem_team_my_pe(ROCSHMEM_TEAM_WORLD);
|
|
n_pes = rocshmem_team_n_pes(ROCSHMEM_TEAM_WORLD);
|
|
|
|
char* value{nullptr};
|
|
if ((value = getenv("ROCSHMEM_TEST_MAX_NUM_TEAMS"))) {
|
|
num_teams = atoi(value);
|
|
} else {
|
|
// Default to number of work groups
|
|
num_teams = args.num_wgs;
|
|
}
|
|
|
|
// Set root PE to 0 by default, can be modified via environment variable
|
|
if ((value = getenv("ROCSHMEM_TEST_BROADCAST_ROOT"))) {
|
|
pe_root = atoi(value);
|
|
if (pe_root < 0 || pe_root >= n_pes) {
|
|
std::cerr << "Invalid ROCSHMEM_TEST_BROADCAST_ROOT value. Using PE 0."
|
|
<< std::endl;
|
|
pe_root = 0;
|
|
}
|
|
}
|
|
|
|
int num_bytes_wg = args.max_msg_size;
|
|
int total_bytes = num_bytes_wg * num_teams;
|
|
buf_size = total_bytes;
|
|
|
|
source_buf = static_cast<char *>(rocshmem_malloc(buf_size));
|
|
dest_buf = static_cast<char *>(rocshmem_malloc(buf_size));
|
|
|
|
if (source_buf == nullptr || dest_buf == nullptr) {
|
|
std::cerr << "Error allocating memory from symmetric heap" << std::endl;
|
|
std::cerr << "source: " << source_buf << ", dest: " << dest_buf
|
|
<< std::endl;
|
|
rocshmem_global_exit(1);
|
|
}
|
|
|
|
team_world_dup.resize(num_teams);
|
|
|
|
streams.resize(num_teams);
|
|
start_events_timed.resize(num_teams);
|
|
stop_events_timed.resize(num_teams);
|
|
for (int i = 0; i < num_teams; i++) {
|
|
CHECK_HIP(hipStreamCreate(&streams[i]));
|
|
CHECK_HIP(hipEventCreate(&start_events_timed[i]));
|
|
CHECK_HIP(hipEventCreate(&stop_events_timed[i]));
|
|
}
|
|
}
|
|
|
|
TeamBroadcastmemOnStreamTester::~TeamBroadcastmemOnStreamTester() {
|
|
for (int i = 0; i < num_teams; i++) {
|
|
CHECK_HIP(hipEventDestroy(stop_events_timed[i]));
|
|
CHECK_HIP(hipEventDestroy(start_events_timed[i]));
|
|
CHECK_HIP(hipStreamDestroy(streams[i]));
|
|
}
|
|
rocshmem_free(source_buf);
|
|
rocshmem_free(dest_buf);
|
|
}
|
|
|
|
void TeamBroadcastmemOnStreamTester::preLaunchKernel() {
|
|
bw_factor = 1; // Broadcast is one-to-all
|
|
|
|
for (int team_i = 0; team_i < num_teams; team_i++) {
|
|
team_world_dup[team_i] = ROCSHMEM_TEAM_INVALID;
|
|
rocshmem_team_split_strided(ROCSHMEM_TEAM_WORLD, 0, 1, n_pes, nullptr, 0,
|
|
&team_world_dup[team_i]);
|
|
if (team_world_dup[team_i] == ROCSHMEM_TEAM_INVALID) {
|
|
std::cerr << "Team " << team_i << " is invalid!" << std::endl;
|
|
abort();
|
|
}
|
|
}
|
|
}
|
|
|
|
void TeamBroadcastmemOnStreamTester::postLaunchKernel() {
|
|
// Synchronize all streams to ensure events are recorded
|
|
for (int i = 0; i < num_teams; i++) {
|
|
CHECK_HIP(hipStreamSynchronize(streams[i]));
|
|
}
|
|
|
|
// Get elapsed time for each work group from HIP events
|
|
for (int wg_id = 0; wg_id < num_teams && wg_id < num_timers; wg_id++) {
|
|
float elapsed_time_ms = 0.0f;
|
|
CHECK_HIP(hipEventElapsedTime(&elapsed_time_ms, start_events_timed[wg_id],
|
|
stop_events_timed[wg_id]));
|
|
|
|
// Convert milliseconds to GPU cycles
|
|
// wall_clk_rate is in kHz, so: cycles = ms * wall_clk_rate
|
|
long long int elapsed_cycles = static_cast<long long int>(
|
|
elapsed_time_ms * static_cast<float>(wall_clk_rate));
|
|
|
|
start_time[wg_id] = 0;
|
|
end_time[wg_id] = elapsed_cycles;
|
|
}
|
|
|
|
// Fill remaining timers with zero if num_timers > num_teams
|
|
for (int i = num_teams; i < num_timers; i++) {
|
|
start_time[i] = 0;
|
|
end_time[i] = 0;
|
|
}
|
|
|
|
for (int team_i = 0; team_i < num_teams; team_i++) {
|
|
rocshmem_team_destroy(team_world_dup[team_i]);
|
|
}
|
|
}
|
|
|
|
void TeamBroadcastmemOnStreamTester::resetBuffers(size_t size) {
|
|
// Initialize source buffer on all PEs
|
|
// Each work group has its own portion
|
|
for (int wg_id = 0; wg_id < num_teams; wg_id++) {
|
|
int idx = wg_id * size;
|
|
if (my_pe == pe_root) {
|
|
// Root PE fills its source buffer with broadcast value
|
|
int value = (pe_root + 1) * 100 + wg_id;
|
|
std::memset(source_buf + idx, value, size);
|
|
} else {
|
|
// Non-root PEs source buffer (not used in broadcast)
|
|
std::memset(source_buf + idx, 0xFF, size);
|
|
}
|
|
}
|
|
|
|
// Initialize destination buffer on all PEs
|
|
// Root PE keeps its initial dest value (broadcast doesn't copy to root's
|
|
// dest) Non-root PEs set to 0 (will receive broadcast data)
|
|
for (int wg_id = 0; wg_id < num_teams; wg_id++) {
|
|
int idx = wg_id * size;
|
|
if (my_pe == pe_root) {
|
|
// Root PE's dest buffer stays with a different value
|
|
int root_dest_value = 0xAA;
|
|
std::memset(dest_buf + idx, root_dest_value, size);
|
|
} else {
|
|
std::memset(dest_buf + idx, 0, size);
|
|
}
|
|
}
|
|
}
|
|
|
|
void TeamBroadcastmemOnStreamTester::launchKernel(dim3 gridSize,
|
|
dim3 blockSize,
|
|
int loop,
|
|
size_t size) {
|
|
// Execute warmup iterations (skip)
|
|
for (int i = 0; i < args.skip; i++) {
|
|
for (int wg_id = 0; wg_id < num_teams; wg_id++) {
|
|
char *wg_source = source_buf + wg_id * size;
|
|
char *wg_dest = dest_buf + wg_id * size;
|
|
rocshmem_broadcastmem_on_stream(team_world_dup[wg_id], wg_dest,
|
|
wg_source, size, pe_root, streams[wg_id]);
|
|
}
|
|
}
|
|
|
|
for (int i = 0; i < num_teams; i++) {
|
|
CHECK_HIP(hipStreamSynchronize(streams[i]));
|
|
}
|
|
|
|
for (int i = 0; i < loop; i++) {
|
|
for (int wg_id = 0; wg_id < num_teams; wg_id++) {
|
|
// Record start event for this work group on first iteration
|
|
if (i == 0) {
|
|
CHECK_HIP(hipEventRecord(start_events_timed[wg_id], streams[wg_id]));
|
|
}
|
|
|
|
char *wg_source = source_buf + wg_id * size;
|
|
char *wg_dest = dest_buf + wg_id * size;
|
|
rocshmem_broadcastmem_on_stream(team_world_dup[wg_id], wg_dest,
|
|
wg_source, size, pe_root, streams[wg_id]);
|
|
|
|
// Record stop event for this work group on last iteration
|
|
if (i == loop - 1) {
|
|
CHECK_HIP(hipEventRecord(stop_events_timed[wg_id], streams[wg_id]));
|
|
}
|
|
}
|
|
}
|
|
|
|
num_msgs = (loop + args.skip) * num_teams;
|
|
num_timed_msgs = loop * num_teams;
|
|
}
|
|
|
|
void TeamBroadcastmemOnStreamTester::verifyResults(size_t size) {
|
|
// Verify correctness: after broadcast, non-root PEs receive the broadcast
|
|
// data Root PE's dest buffer is NOT modified (per OpenSHMEM/rocSHMEM spec)
|
|
for (int wg_id = 0; wg_id < num_teams; wg_id++) {
|
|
int idx = wg_id * size;
|
|
int expected_value;
|
|
|
|
if (my_pe == pe_root) {
|
|
// Root PE's dest buffer should remain unchanged (0xAA)
|
|
expected_value = 0xAA;
|
|
} else {
|
|
// Non-root PEs should have received the broadcast value
|
|
expected_value = (pe_root + 1) * 100 + wg_id;
|
|
}
|
|
|
|
for (size_t k = 0; k < size; k++) {
|
|
if (static_cast<unsigned char>(dest_buf[idx + k]) !=
|
|
static_cast<unsigned char>(expected_value)) {
|
|
std::cerr << "PE " << my_pe << ": Verification failed for WG "
|
|
<< wg_id << " at byte " << k << std::endl;
|
|
std::cerr << "Expected value: " << expected_value
|
|
<< ", Got: " << static_cast<int>(dest_buf[idx + k])
|
|
<< std::endl;
|
|
rocshmem_global_exit(1);
|
|
}
|
|
}
|
|
}
|
|
}
|
|
|